Module: Unicorn
- Defined in:
- lib/unicorn.rb,
lib/unicorn/oob_gc.rb,
lib/unicorn/cgi_wrapper.rb,
lib/unicorn/app/old_rails.rb,
lib/unicorn/preread_input.rb,
lib/unicorn/socket_helper.rb
Overview
Unicorn module containing all of the classes (include C extensions) for running a Unicorn web server. It contains a minimalist HTTP server with just enough functionality to service web application requests fast as possible.
Defined Under Namespace
Modules: App, Const, HttpResponse, Launcher, SocketHelper, Util Classes: CGIWrapper, ClientShutdown, Configurator, HttpParser, HttpParserError, HttpServer, OobGC, PrereadInput, TeeInput, TmpIO, Worker
Class Method Summary collapse
-
.builder(ru, opts) ⇒ Object
This returns a lambda to pass in as the app, this does not "build" the app (which we defer based on the outcome of "preload_app" in the Unicorn config).
-
.listener_names ⇒ Object
returns an array of strings representing TCP listen socket addresses and Unix domain socket paths.
- .run(app, options = {}) ⇒ Object
Class Method Details
.builder(ru, opts) ⇒ Object
This returns a lambda to pass in as the app, this does not "build" the app (which we defer based on the outcome of "preload_app" in the Unicorn config). The returned lambda will be called when it is time to build the app.
20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 |
# File 'lib/unicorn.rb', line 20 def self.builder(ru, opts) # allow Configurator to parse cli switches embedded in the ru file Unicorn::Configurator::RACKUP.update(:file => ru, :optparse => opts) # always called after config file parsing, may be called after forking lambda do || inner_app = case ru when /\.ru$/ raw = File.read(ru) raw.sub!(/^__END__\n.*/, '') eval("Rack::Builder.new {(#{raw}\n)}.to_app", TOPLEVEL_BINDING, ru) else require ru Object.const_get(File.basename(ru, '.rb').capitalize) end pp({ :inner_app => inner_app }) if $DEBUG # return value, matches rackup defaults based on env case ENV["RACK_ENV"] when "development" Rack::Builder.new do use Rack::CommonLogger, $stderr use Rack::ShowExceptions use Rack::Lint run inner_app end.to_app when "deployment" Rack::Builder.new do use Rack::CommonLogger, $stderr run inner_app end.to_app else inner_app end end end |
.listener_names ⇒ Object
returns an array of strings representing TCP listen socket addresses and Unix domain socket paths. This is useful for use with Raindrops::Middleware under Linux: http://raindrops.bogomips.org/
61 62 63 64 65 |
# File 'lib/unicorn.rb', line 61 def self.listener_names Unicorn::HttpServer::LISTENERS.map do |io| Unicorn::SocketHelper.sock_name(io) end end |
.run(app, options = {}) ⇒ Object
12 13 14 |
# File 'lib/unicorn.rb', line 12 def self.run(app, = {}) Unicorn::HttpServer.new(app, ).start.join end |